package ru.ldeloff.servermonitorbot.service;

import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import ru.ldeloff.servermonitorbot.config.SshConfig;
import ru.ldeloff.servermonitorbot.model.SshServer;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class SshServiceImpl implements SshService{

    private final List<SshServer> sshServers;

    @Autowired
    public SshServiceImpl(SshConfig sshConfig) {
        this.sshServers = sshConfig.getServers();
    }

    private List<Session> sessions = new ArrayList<>();

    @Override
    public void connectToAllServer() {
        sshServers.forEach(sshServer -> {
            try {
                Session session = connectToServer(sshServer);
                sshServer.setSession(session);
                log.info("Успешно подключён к " + sshServer.getHost());
                sessions.add(session);
            } catch (JSchException e) {
                log.warn("Не удалось соединиться с " + sshServer.getHost() + ": " + e.getMessage());
            }
        });
    }

    @Override
    public Session connectToServer(SshServer sshServer) throws JSchException {
        Session session = new JSch().getSession(sshServer.getUser(),
                sshServer.getHost(),
                sshServer.getPort());
        session.setPassword(sshServer.getPassword());
        session.setConfig("StrictHostKeyChecking", "no");
        session.connect();
        return session;
    }

    @Override
    public void disconnectSessions() {
        if (!sessions.isEmpty()) {
            sessions.forEach(Session::disconnect);
            sessions.forEach(sessions::remove);
        }
    }

    @Override
    public SendMessage getStatusSessions(Long chatId) {
        StringBuilder text = new StringBuilder("Статус соединения: \n");
        sshServers.forEach(server -> {
            text.append(server.getName())
                    .append(": ")
                    .append(server.getSession().isConnected() ? "OK" : "отключён")
                    .append("\n");
        });
        SendMessage message = new SendMessage();
        message.setText(String.valueOf(text));
        message.setChatId(chatId);
        return message;
    }
}
