package ru.ldeloff.servermonitorbot.service;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.bots.TelegramLongPollingBot;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.api.objects.Update;
import org.telegram.telegrambots.meta.exceptions.TelegramApiException;
import ru.ldeloff.servermonitorbot.model.TelegramBot;
import ru.ldeloff.servermonitorbot.util.TelegramBotMenu;

@Service
@Slf4j
public class TelegramBotService extends TelegramLongPollingBot {

    final TelegramBot telegramBot;
    final SshService sshService;
    final TelegramBotMenu telegramBotMenu;


    @Autowired
    public TelegramBotService(TelegramBot telegramBot, SshService sshService, TelegramBotMenu telegramBotMenu) {
        this.telegramBot = telegramBot;
        this.sshService = sshService;
        this.telegramBotMenu = telegramBotMenu;
    }

    @Override
    public void onUpdateReceived(Update update) {
        if (update.hasMessage()) {
            if (update.getMessage().hasText()) {
                SendMessage message = new SendMessage();
                String messageText = update.getMessage().getText();
                long chatId = update.getMessage().getChatId();
                switch (messageText) {
                    case "/start":
                        message.setText("Привет: " + update.getMessage().getChat().getUserName());
                        message.setChatId(chatId);
                        message.setReplyMarkup(telegramBotMenu.initKeyboard());
                        sendMessage(message);
                        log.info("Получена команда /start");
                        break;
                    case "Статус":
                        sendMessage(sshService.getStatusSessions(chatId));
                        log.info("Получена команда 'Статус'");
                        break;
                    default:
                        message.setText("unknown command:" + messageText);
                        message.setChatId(chatId);
                        message.setReplyMarkup(telegramBotMenu.initKeyboard());
                        sendMessage(message);
                        log.warn("Неизвестная команда:" + messageText);
                }
            }
        } else if (update.hasCallbackQuery()) {
            try {
                SendMessage message = new SendMessage();
                message.setText(update.getCallbackQuery().getData());
                message.setChatId(update.getCallbackQuery().getMessage().getChatId());
                execute(message);
            } catch (TelegramApiException e) {
                e.printStackTrace();
            }
        }
    }

    private void sendMessage(SendMessage message) {
        try {
            execute(message);
        } catch (TelegramApiException e) {
            e.printStackTrace();
            log.warn(e.getMessage());
        }
    }

    @Override
    public String getBotUsername() {
        return telegramBot.getBotUsername();
    }

    @Override
    public String getBotToken() {
        return telegramBot.getBotToken();
    }

    @Override
    public void onClosing() {
        super.onClosing();
        sshService.disconnectSessions();
    }
}
